/* HelpDialog.cpp
 * Copyright (C) 2001 Dustin Graves <dgraves@computer.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "fox/fx.h"
#include "HelpDialog.h"

const FXchar about_txt[]=\
"What?\n"\
"=====\n"\
"MediSen is an administrative tool for enabling and disabling the Windows 2000 Media Sensing feature.  "\
"It indicates the current status of the Media Sensing feature for a given workstation and allows for the "\
"modification of that status.  MediSen can be used to administer Windows 2000 workstations both locally "\
"and remotely.  At the time this application was written, it was necessary to disable the Media Sensing "\
"feature in order to send multicast datagrams on the loopback address of a Windows 2000 workstation whose "\
"network adapter was not connected to a network.\n"\
"\n"\
"\n"\
"Why?\n"\
"====\n"\
"Windows 2000 contains a new network related feature that can detect whether an active network connection "\
"is present thorough an installed network interface card.  If Windows 2000 detects a \"down\" state for a "\
"network interface card it removes bound protocols from that adapter until it is detected to be \"up\" again.  "\
"This, apparently, is intended for multihomed systems whose network media are not always in an \"up\" state "\
"(when one interface goes down, it's associated routes are removed, and all outgoing traffic is directed through "\
"another interface, if that interface is the default gateway).  This also has the unfortunate side effect of "\
"making it impossible to join a multicast group, even when sending and receiving on the loopback adapter.  "\
"Any attempt to join a multicast group will fail with winsock error 10049 (WSAEADDRNOTAVAIL) indicating that "\
"the requested address could not be assigned.\n"\
"I have discussed this problem with Microsoft Support (case SRZ010305000785), which does not consider this to "\
"be an implementation error.  During this discussion I was presented with two work-arounds for this issue "\
"which are to:\n"\
"\n"\
"\t*Disable the Media Sensing feature\n"\
"\t*Install a Virtual Network Device\n"\
"\n"\
"There is also a third option of keeping your workstation plugged into a powered hub at all times in order to "\
"fool Windows 2000 into thinking that the network adapter has an active link, but this is not very practical "\
"for laptops and portable systems that do not always have ready access to an electrical outlet required to "\
"power a hub.\n"\
"MediSen implements the first option of disabling the Media Sensing feature via a registry setting.  More "\
"information regarding editing the registry to disable the Media Sensing feature can be found here:\n"\
"\thttp://support.microsoft.com/support/kb/articles/q239/9/24.asp\n"\
"\n"\
"The second option of installing a \"Virtual Network Device\" is a simple solution that is easily done.  "\
"In order to install this device, double click the Add/Remove Hardware item in the control panel, choose to "\
"add a device, choose to pick the device from a list, choose Microsoft from the vendor list, and then select "\
"Microsoft Loopback Adapter from the device list.  This will install a second loopback adapter on your "\
"Windows 2000 workstation (your network adapter also has one, but it is not usable for multicast when the "\
"network is \"down\").  When the protocols for a \"down\" network adapter are unbound, all multicast traffic "\
"will be routed through this secondary loopback adapter.  This is a very viable solution, however, at the time "\
"that this application was written there was a bug with the Microsoft Loopback Adapter that resulted in the "\
"Microsoft Loopback Adapter not actually routing incoming multicast datagrams to the loopback address "\
"(typically \"127.0.0.1\").  The datagrams are being sent, but where they are actually being sent to is a "\
"mystery.  To make an application send multicast datagrams to the loopback address via the Microsoft Loopback "\
"Adapter, the application needs to be explicitly set to route outgoing traffic to the \"127.0.0.1\" "\
"interface.  This could require a modification to an existing multicast application, which negates the "\
"viability of this option.  Microsoft Support has informed me that there will be a \"hot fix\" for this bug "\
"in the Windows 2000 Service Pack 2.\n"\
"\n"\
"At the current time, I've found disabling the Media Sensing feature to be the best solution to the problem, "\
"and that writing a simple application to edit the registry for the local and remote registry(s) makes this "\
"easier for a network administrator to accomplish this task.  Because MediSen is capable of remote registry "\
"administration, it is possible for a network administrator to disable the Media Sensing feature for all "\
"machines on a network from a single workstation.\n"\
"\n"\
"\n"\
"Who?\n"\
"====\n"\
"MediSen is Copyright (C) 2001 Dustin Graves <dgraves@computer.org>\n";

const FXchar doc_txt[]=\
"Usage\n"\
"=====\n"\
"MediSen is a fairly simple application.  It provides the user with two menu options for exiting the "\
"application and connecting to a remote registry, a label indicating the Media Sensing status for a given "\
"workstation, a set of radio buttons for disabling/enabling the Media Sensing Feature for a given "\
"workstation, and two buttons for applying changes to a given workstation and exiting the application.  "\
"An individual description of an item may be obtained through MediSen by right clicking the item.\n"\
"\n"\
"*Please note that Administrative priviledges may be required to use this software.*\n"\
"\n"\
"Using the pull down menus\n"\
"========================\n"\
"The two pull down menus each have one option.  The \"File\" menu has an option to exit the application.  "\
"When this option is selected the application will quit.  If a modification has been made to the Media "\
"Sensing registry setting, the user will be given the choice of rebooting the workstation to allow these "\
"changes to take effect.\n"\
"The \"Options\" menu has an option to connect to a remote network registry.  When this option is selected, "\
"the user will be presented with a dialog for entering the name of the remote workstation to connect to.  "\
"The user may either cancel the operation, enter the computer name by hand, or select a workstation from the "\
"network neighborhood with the \"Browse\" option.  Loading the contents of the Network Neighborhood and "\
"connecting to a remote registry can sometimes be a slow procedure.\n"\
"\n"\
"If using MediSen on Windows 9x to remotely administer Windows 2000 workstations, in some situations, "\
"connecting to a remote workstation's registry may fail with the error ERROR_DLL_INIT_FAILED indicating "\
"that \"A dynamic link library (DLL) initialization routine failed.\"  In order to obtain remote access "\
"to a registry from Windows 9x you must install the Microsoft Remote Registry Service.  Here is the "\
"procedure for doing this with Windows 95.\n"\
"\n"\
"Windows 95: To use RegConnectRegistry on Windows 95, you must install the Microsoft Remote Registry "\
"service. If you don't do this, calls to RegConnectRegistry return ERROR_DLL_INIT_FAILED. Perform the "\
"following steps to install the Microsoft Remote Registry service.\n"\
"To install the Microsoft Remote Registry service\n"\
"\n"\
"\t1. In the Network option in Control Panel, click Add.\n"\
"\t2. In the Select Network Component Type dialog box, double-click Service.\n"\
"\t3. In the Select Network Service dialog box, click the Have Disk button.\n"\
"\t4. In the Install From Disk dialog box, type the path to the ADMIN\\NETTOOLS\\REMOTREG "\
"directory on the Windows 95 compact disc, and then click OK.\n"\
"\t5. In the Install From Disk dialog box, click OK.\n"\
"\t6. In the Select Network Service dialog box, click Microsoft Remote Registry, and then click OK.\n"\
"\n"\
"The procedure for Windows 98 is the same except that step 4 is as follows:\n"\
"\n"\
"\t4. In the Install From Disk dialog box, type the path to the TOOLS\\RESKIT\\NETADMIN\\REMOTREG "\
"directory on the Windows 95 compact disc, and then click OK.\n"\
"\n"\
"Making Sense of the Media Sensing Status Report\n"\
"===============================================\n"\
"MediSen will display one of the following three status messages to indicate the Media Sensing status "\
"for a given host:\n"\
"\n"\
"\t1.  Media Sensing feature is not currently disabled for <hostname>\n"\
"\t2.  Media Sensing feature is currently enabled for <hostname>\n"\
"\t3.  Media Sensing feature is currently disabled for <hostname>\n"\
"\n"\
"<hostname> indicates the name of the host that MediSen is connected to.  Option number one indicates that "\
"there is currently no setting in the registry to either enable or disable the Media Sensing feature.  "\
"In this case the Media Sensing feature will default to on.  Option number two indicates that the Media "\
"Sensing registry value is set to true, indicating that the Media Sensing feature is currently on.  "\
"Option number three indicates that the Media Sensing registry value is set to false, indicating that "\
"the Media Sensing feature is currently off.\n"\
"\n"\
"Making Changes to the Media Sensing Setting\n"\
"===========================================\n"\
"The status of the Media Sensing feature can be changed by selecting the appropriate radio button and "\
"applying the change.  Selecting the radio button labeled \"Enable\" indicates that the Media Sensing "\
"feature is to be turned on, and selecting the radio button labeled \"Disable\" indicates that the "\
"Media Sensing feature is to be turned off.  This selection will then need to be activated with the "\
"button labeled \"Apply.\"\n"\
"A disabled apply button indicates that the current radio button choice matches the current Media Sensing "\
"registry setting, and no changes need to be applied.\n";

FXIMPLEMENT(HelpDialog,FXDialogBox,NULL,0)

HelpDialog::HelpDialog(FXWindow* owner,const FXString& title)
: FXDialogBox(owner,title,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0, 4,4,4,4, 4,4)
{
  FXHorizontalFrame* buttons=new FXHorizontalFrame(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);
  new FXButton(buttons,"&Close",NULL,this,FXDialogBox::ID_ACCEPT,LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 20,20,4,4);

  FXVerticalFrame* tabframe=new FXVerticalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
  FXTabBook*tabbook=new FXTabBook(tabframe,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);

  FXTabItem* abouttab=new FXTabItem(tabbook,"&Background",NULL);
  FXVerticalFrame* aboutframe=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
  FXVerticalFrame* aboutsink=new FXVerticalFrame(aboutframe,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
  FXText* abouttext=new FXText(aboutsink,NULL,0,TEXT_READONLY|TEXT_WORDWRAP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  abouttext->setVisRows(50);
  abouttext->setVisCols(60);
  abouttext->setText(about_txt);

  FXTabItem* doctab=new FXTabItem(tabbook,"&Usage",NULL);
  FXVerticalFrame* docframe=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
  FXVerticalFrame* docsink=new FXVerticalFrame(docframe,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
  FXText* doctext=new FXText(docsink,NULL,0,TEXT_READONLY|TEXT_WORDWRAP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  doctext->setVisRows(50);
  doctext->setVisCols(60);
  doctext->setText(doc_txt);
}
