/* MediSen.cpp
 * Copyright (C) 2001 Dustin Graves <dgraves@computer.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include <windows.h>
#include <winreg.h>
#include "fox/fx.h"
#include "icons.h"
#include "HelpDialog.h"
#include "FXNetworkDialogEx.h"

//Reboot dialog
class MediSen;

class RebootDialog : public FXDialogBox
{
  FXDECLARE(RebootDialog)
protected:
  FXbool showReboot;

  RebootDialog(){}

public:
  RebootDialog(MediSen*,const FXString&,FXbool=TRUE);

  virtual void create();
};

//Remote Host Dialog
class RemoteDialog : public FXDialogBox
{
  FXDECLARE(RemoteDialog)
protected:
  FXTextField* remoteHost;

  RemoteDialog(){}

public:
  RemoteDialog(MediSen*);
  FXString getRemoteHost() const;

  virtual void create();
  long onCmdBrowse(FXObject*,FXSelector,void*);

  enum
  {
    ID_BROWSE=FXDialogBox::ID_LAST,
    ID_LAST
  };
};

enum
{
  VALUE_DNE=0,
  VALUE_ENABLED=1,
  VALUE_DISABLED=2
};

class MediSen : public FXMainWindow
{
  FXDECLARE(MediSen)

protected:
  FXuint value;
  FXbool enable;
  FXbool dirty;
  FXbool clarify;
  FXString host;
  FXStatusbar* statusbar;
  FXMenuPane* file;
  FXMenuPane* options;
  FXMenuPane* help;
  FXIcon* bigIcon;
  FXIcon* miniIcon;

  MediSen(){}
  void showErrorMessage(FXint);
  FXint checkRegistryValue();
  FXbool rebootSystem();

public:
  MediSen(FXApp*);
  ~MediSen();

  virtual void create();

  long onCmdQuit(FXObject*,FXSelector,void*);
  long onCmdConnect(FXObject*,FXSelector,void*);
  long onCmdChoice(FXObject*,FXSelector,void*);
  long onUpdChoice(FXObject*,FXSelector,void*);
  long onUpdStatus(FXObject*,FXSelector,void*);
  long onCmdApply(FXObject*,FXSelector,void*);
  long onUpdApply(FXObject*,FXSelector,void*);
  long onCmdHelp(FXObject*,FXSelector,void*);
  long onCmdAbout(FXObject*,FXSelector,void*);
  long onPopupMenu(FXObject*,FXSelector,void*);
  long onCmdClarify(FXObject*,FXSelector,void*);

  enum
  {
    ID_QUIT=FXMainWindow::ID_LAST,
    ID_CONNECT,
    ID_MSENABLE,
    ID_MSDISABLE,
    ID_STATUS,
    ID_APPLY,
    ID_CANCEL,
    ID_HELP,
    ID_ABOUT,
    ID_CLARIFY,
    ID_LAST
  };
};

#define LOCALHOST "local host"
#define EDITSTRING(x) (FXString("Editing registry for ")+FXString(x))
#define DNESTRING(x) (FXString("Media Sensing feature is not currently disabled for ")+FXString(x))
#define ENABLEDSTRING(x) (FXString("Media Sensing feature is currently enabled for ")+FXString(x))
#define DISABLEDSTRING(x) (FXString("Media Sensing feature is currently disabled for ")+FXString(x))

FXIMPLEMENT(RebootDialog,FXDialogBox,NULL,0)

RebootDialog::RebootDialog(MediSen* owner,const FXString& sysname,FXbool showReboot)
: FXDialogBox(owner,"Reboot System",DECOR_TITLE|DECOR_BORDER,0,0,0,0, 10,10,10,10, 4,4)
{
  FXString title=FXString("Reboot ")+sysname.text();
  if(sysname!=LOCALHOST)
    handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),&title);

  FXVerticalFrame* contents=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y);

  if(sysname!=LOCALHOST)
    title=sysname.text()+FXString(" must be rebooted for changes to take effect.");
  else
    title="The system must be rebooted for changes to take effect.";
  new FXLabel(contents,title,NULL,JUSTIFY_LEFT);

  if(showReboot)
  {
    if(sysname!=LOCALHOST)
      title=FXString("would you like ")+sysname.text()+FXString(" to reboot now?");
    else
      title="Would you like the system to reboot now?";
    new FXLabel(contents,title);
  }

  new FXFrame(contents,LAYOUT_FIX_HEIGHT,0,0,0,8);
  FXHorizontalFrame* buttons=new FXHorizontalFrame(contents,PACK_UNIFORM_WIDTH|LAYOUT_CENTER_X,0,0,0,0, 0,0,0,0);
  if(showReboot)
  {
    new FXButton(buttons,"&Now",NULL,this,ID_ACCEPT,FRAME_THICK|FRAME_RAISED,0,0,0,0,20,20);
    new FXButton(buttons,"&Later",NULL,this,ID_CANCEL,FRAME_THICK|FRAME_RAISED,0,0,0,0,20,20);
  }
  else
    new FXButton(buttons,"&OK",NULL,this,ID_CANCEL,FRAME_THICK|FRAME_RAISED,0,0,0,0,20,20);
}

void RebootDialog::create()
{
  FXDialogBox::create();
}

FXDEFMAP(RemoteDialog) RemoteDialogMap[]={
  FXMAPFUNC(SEL_COMMAND,RemoteDialog::ID_BROWSE,RemoteDialog::onCmdBrowse)
};

FXIMPLEMENT(RemoteDialog,FXDialogBox,RemoteDialogMap,ARRAYNUMBER(RemoteDialogMap))

RemoteDialog::RemoteDialog(MediSen* owner)
: FXDialogBox(owner,"Connect Network Registry",DECOR_TITLE|DECOR_BORDER,0,0,0,0, 10,10,10,10, 4,4)
{
  FXVerticalFrame* contents=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y);
  FXMatrix* matrix=new FXMatrix(contents,4,MATRIX_BY_ROWS|LAYOUT_FILL_X,0,0,0,0, 0,0,0,0, 10);
  new FXLabel(matrix,"Computer Name:",NULL,JUSTIFY_LEFT|LAYOUT_FILL_ROW);
  remoteHost=new FXTextField(matrix,20,this,ID_ACCEPT,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXFrame(matrix,0);
  new FXFrame(matrix,0);
  new FXButton(matrix,"&OK",NULL,this,ID_ACCEPT,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,20,20);
  new FXButton(matrix,"&Cancel",NULL,this,ID_CANCEL,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,20,20);
  new FXFrame(matrix,0);
  new FXButton(matrix,"&Browse...",NULL,this,ID_BROWSE,FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X);
}

void RemoteDialog::create()
{
  FXDialogBox::create();
}

FXString RemoteDialog::getRemoteHost() const
{
  return remoteHost->getText();
}

long RemoteDialog::onCmdBrowse(FXObject*,FXSelector,void*)
{
  //Popup network browsing dialog
  FXNetworkDialogEx dlg(this,"Browse for Computer");
  if(dlg.execute(PLACEMENT_CURSOR,TRUE))
  {
    remoteHost->setText(dlg.getSelection());
  }
  return 1;
}

FXDEFMAP(MediSen) MediSenMap[]={
  FXMAPFUNC(SEL_CLOSE,0,MediSen::onCmdQuit),
  FXMAPFUNC(SEL_COMMAND,MediSen::ID_QUIT,MediSen::onCmdQuit),
  FXMAPFUNC(SEL_COMMAND,MediSen::ID_CONNECT,MediSen::onCmdConnect),
  FXMAPFUNCS(SEL_COMMAND,MediSen::ID_MSENABLE,MediSen::ID_MSDISABLE,MediSen::onCmdChoice),
  FXMAPFUNCS(SEL_UPDATE,MediSen::ID_MSENABLE,MediSen::ID_MSDISABLE,MediSen::onUpdChoice),
  FXMAPFUNC(SEL_UPDATE,MediSen::ID_STATUS,MediSen::onUpdStatus),
  FXMAPFUNC(SEL_COMMAND,MediSen::ID_APPLY,MediSen::onCmdApply),
  FXMAPFUNC(SEL_UPDATE,MediSen::ID_APPLY,MediSen::onUpdApply),
  FXMAPFUNC(SEL_COMMAND,MediSen::ID_CANCEL,MediSen::onCmdQuit),
  FXMAPFUNC(SEL_COMMAND,MediSen::ID_HELP,MediSen::onCmdHelp),
  FXMAPFUNC(SEL_COMMAND,MediSen::ID_ABOUT,MediSen::onCmdAbout),
  FXMAPFUNCS(SEL_RIGHTBUTTONRELEASE,MediSen::ID_MSENABLE,MediSen::ID_CANCEL,MediSen::onPopupMenu),
  FXMAPFUNC(SEL_COMMAND,MediSen::ID_CLARIFY,MediSen::onCmdClarify)
};

FXIMPLEMENT(MediSen,FXMainWindow,MediSenMap,ARRAYNUMBER(MediSenMap))

MediSen::MediSen(FXApp* a)
: FXMainWindow(a,"MediSen",NULL,NULL,DECOR_ALL,0,0,0,0, 0,0,0,0, 0,0),
  value(VALUE_DNE),
  enable(TRUE),
  dirty(FALSE),
  clarify(FALSE),
  host(LOCALHOST)
{
  bigIcon=new FXBMPIcon(getApp(),medisen_big,0,IMAGE_ALPHAGUESS);
  miniIcon=new FXBMPIcon(getApp(),medisen_small,0,IMAGE_ALPHAGUESS);
  setIcon(bigIcon);
  setMiniIcon(miniIcon);

  FXMenubar* menubar=new FXMenubar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
  file=new FXMenuPane(this);
  new FXMenuCommand(file,"E&xit\tCtrl+X\tExit Media Sensing Controller",NULL,this,ID_QUIT);
  new FXMenuTitle(menubar,"&File",NULL,file);
  options=new FXMenuPane(this);
  new FXMenuCommand(options,"&Connect Network Registry...\tCtrl+R\tEdit Registry on a Remote Host",NULL,this,ID_CONNECT);
  new FXMenuTitle(menubar,"&Options",NULL,options);
  help=new FXMenuPane(this);
  new FXMenuCommand(help,"&Help\tCtrl+H\tDisplay Help Dialog",NULL,this,ID_HELP);
  new FXMenuSeparator(help);
  new FXMenuCommand(help,"&About\tCtrl+A\tDisplay About Dialog",NULL,this,ID_ABOUT);
  new FXMenuTitle(menubar,"&Help",NULL,help,LAYOUT_RIGHT);
  new FXHorizontalSeparator(this,SEPARATOR_GROOVE|LAYOUT_SIDE_TOP|LAYOUT_FILL_X);

  statusbar=new FXStatusbar(this,STATUSBAR_WITH_DRAGCORNER|LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X);
  statusbar->getStatusline()->setNormalText(FXString(EDITSTRING(host)));

  FXVerticalFrame* contents=new FXVerticalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);

  new FXFrame(contents,LAYOUT_FIX_HEIGHT,0,0,0,4);
  FXVerticalFrame* labels=new FXVerticalFrame(contents,LAYOUT_FILL_X,0,0,0,0, 0,0,0,0, 0,0);
  new FXLabel(labels,"Status: ",NULL);
  FXLabel* status=new FXLabel(labels,DNESTRING(host),NULL,JUSTIFY_LEFT);
  status->enable();
  status->setTarget(this);
  status->setSelector(ID_STATUS);
  new FXFrame(contents,LAYOUT_FIX_HEIGHT,0,0,0,4);

  FXGroupBox* choices=new FXGroupBox(contents,"Media Sensing Choices",GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXRadioButton(choices,"Enable \"Media Sensing\" feature",this,ID_MSENABLE);
  new FXRadioButton(choices,"Disable \"Media Sensing\" feature",this,ID_MSDISABLE);

  FXHorizontalFrame* buttons=new FXHorizontalFrame(contents,LAYOUT_RIGHT|PACK_UNIFORM_WIDTH,0,0,0,0, 0,0,0,0);
  new FXButton(buttons,"&Apply\tApply Setting\tApply change to \"Media Sensing\" setting",NULL,this,ID_APPLY,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,20,20);
  new FXButton(buttons,"&Exit\tExit Without Update\tExit without applying change to \"Media Sensing\" setting",NULL,this,ID_CANCEL,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,20,20);
}

MediSen::~MediSen()
{
  delete help;
  delete options;
  delete file;
  delete miniIcon;
  delete bigIcon;
}

void MediSen::create()
{
  FXMainWindow::create();
  FXint success=checkRegistryValue();
  if(success==ERROR_SUCCESS)
    enable=(value==VALUE_DISABLED)?TRUE:FALSE;
  else
    showErrorMessage(success);

  show(PLACEMENT_CURSOR);
}

void MediSen::showErrorMessage(FXint error)
{
  LPVOID msg;
  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|
                FORMAT_MESSAGE_FROM_SYSTEM|
                FORMAT_MESSAGE_IGNORE_INSERTS,
                NULL,error,
                MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
                (LPSTR)&msg,0,NULL);

  //Get string and remove any trailing \n\r sequences
  FXString str((char*)msg);
  str.trimEnd();
  FXMessageBox::error(this,MBOX_OK,"Error","Error %d: %s\n",error,str.text());
  LocalFree(msg);
}

FXint MediSen::checkRegistryValue()
{
  HKEY hkey=0;
  HKEY hsubkey=0;

  //Open the handle
  FXint success=0;
  if(host==LOCALHOST)
    success=RegConnectRegistry(NULL,HKEY_LOCAL_MACHINE,&hkey);
  else
  {
    FXString str=FXString("\\\\")+host.text();
    success=RegConnectRegistry(str.text(),HKEY_LOCAL_MACHINE,&hkey);
  }

  if(success!=ERROR_SUCCESS)
    return success;

  success=RegOpenKeyEx(hkey,"System\\CurrentControlSet\\Services\\Tcpip\\Parameters",0,KEY_QUERY_VALUE,&hsubkey);
  if(success==ERROR_FILE_NOT_FOUND)
  {
    //The key is not there.  Probably not Win2k.  Maybe add a warning here?
    value=VALUE_DNE;
    RegCloseKey(hkey);
    return ERROR_SUCCESS;
  }
  if(success!=ERROR_SUCCESS)
  {
    RegCloseKey(hkey);
    return success;
  }

  DWORD val=0;
  DWORD size=sizeof(DWORD);
  success=RegQueryValueEx(hsubkey,"DisableDHCPMediaSense",0,NULL,(LPBYTE)&val,&size);
  if(success==ERROR_FILE_NOT_FOUND)
    value=VALUE_DNE;
  else if(success!=ERROR_SUCCESS)
  {
    RegCloseKey(hsubkey);
    RegCloseKey(hkey);
    return success;
  }
  else
  {
    if(val==0)
      value=VALUE_ENABLED;
    else
      value=VALUE_DISABLED;
  }

  RegCloseKey(hsubkey);
  RegCloseKey(hkey);

  return ERROR_SUCCESS;
}

FXbool MediSen::rebootSystem()
{
  FXbool canReboot=FALSE;

  //Do we actually have the priveledge?
  HANDLE token=0;
  BOOL result=OpenProcessToken(GetCurrentProcess(),TOKEN_ADJUST_PRIVILEGES|TOKEN_QUERY,&token);
  if(result!=0)
  {
    TOKEN_PRIVILEGES priv;
    result=LookupPrivilegeValue((host==LOCALHOST)?NULL:host.text(),
                                (host==LOCALHOST)?SE_SHUTDOWN_NAME:SE_REMOTE_SHUTDOWN_NAME,
                                &priv.Privileges[0].Luid);
    if(result!=0)
    {
      priv.PrivilegeCount=1;
      priv.Privileges[0].Attributes=SE_PRIVILEGE_ENABLED;
      AdjustTokenPrivileges(token,FALSE,&priv,0,0,0);
      if(GetLastError()==ERROR_SUCCESS)
        canReboot=TRUE;
    }
  }
  
  //Did one of the previous calls return this error?
  if(GetLastError()==ERROR_CALL_NOT_IMPLEMENTED)
  {
    //This usually indicates that we are using Win9x so we can do it
    canReboot=TRUE;
  }

  RebootDialog dlg(this,host,canReboot);
  dlg.create();
  if(dlg.execute())
  {
    FXint success=0;
    if(host==LOCALHOST)
      success=ExitWindowsEx(EWX_REBOOT,0);
    else
      success=InitiateSystemShutdown((LPTSTR)host.text(),"The Windows Media Sensing configuration is being modified.  The system must reboot to complete the operation.  ",30,FALSE,TRUE);

    if(success==0)
    {
      showErrorMessage(GetLastError());
      return FALSE;  //Did not reboot
    }
    return TRUE;  //Did reboot
  }

  return (canReboot)?FALSE:TRUE;  //Did not reboot
}

long MediSen::onCmdQuit(FXObject*,FXSelector,void*)
{
  //Inquire about reboot
  if(dirty)
    rebootSystem();

  getApp()->stop();
  return 1;
}

long MediSen::onCmdConnect(FXObject*,FXSelector,void*)
{
  RemoteDialog dlg(this);
  if(dlg.execute())
  {
    FXString str=dlg.getRemoteHost();

    //Compare with current host name
    char localname[MAX_COMPUTERNAME_LENGTH+1];
    DWORD localnamelen=MAX_COMPUTERNAME_LENGTH+1;
    GetComputerName(localname,&localnamelen);
    if(comparecase(str,localname)==0)
    {
      FXMessageBox::error(this,MBOX_OK,"Error Connecting Network Registry","You can not connect to your own computer.");
    }
    else
    {
      FXString prev=host;
      host=str;

      getApp()->beginWaitCursor();
      getApp()->flush();
      FXint success=checkRegistryValue();
      getApp()->endWaitCursor();
      if(success==ERROR_SUCCESS)
      {
        enable=(value==VALUE_DISABLED)?TRUE:FALSE;
        statusbar->getStatusline()->setNormalText(FXString(EDITSTRING(host)));
        dirty=FALSE;
      }
      else
      {
	showErrorMessage(success);
	host=prev;
      }
    }
  }

  return 1;
}

long MediSen::onCmdChoice(FXObject*,FXSelector sel,void*)
{
  if(SELID(sel)==ID_MSENABLE)
    enable=TRUE;
  else
    enable=FALSE;
  return 1;
}

long MediSen::onUpdChoice(FXObject* sender,FXSelector sel,void*)
{
  FXint id=SELID(sel);
  if((id==ID_MSENABLE)&&enable)
    sender->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
  else if((id==ID_MSDISABLE)&&(!enable))
    sender->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
  else
    sender->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);

  return 1;
}

long MediSen::onUpdStatus(FXObject* sender,FXSelector,void*)
{
  FXString str=DNESTRING(host);

  checkRegistryValue();

  if(value==VALUE_ENABLED)
    str=ENABLEDSTRING(host);
  else if(value==VALUE_DISABLED)
    str=DISABLEDSTRING(host);

  sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&str);

  return 1;
}

long MediSen::onCmdApply(FXObject*,FXSelector,void*)
{
  HKEY hkey=0;
  HKEY hsubkey=0;

  //Open the handle
  FXint success=0;
  if(host==LOCALHOST)
    success=RegConnectRegistry(NULL,HKEY_LOCAL_MACHINE,&hkey);
  else
  {
    FXString str=FXString("\\\\")+host.text();
    success=RegConnectRegistry(str.text(),HKEY_LOCAL_MACHINE,&hkey);
  }

  if(success!=ERROR_SUCCESS)
  {
    showErrorMessage(success);
    return 1;
  }

  success=RegOpenKeyEx(hkey,"System\\CurrentControlSet\\Services\\Tcpip\\Parameters",0,KEY_SET_VALUE,&hsubkey);
  if(success==ERROR_FILE_NOT_FOUND)
  {
    //The key is not there.  Probably not Win2k.  Warn them.  
    FXint reply=FXMessageBox::question(this,MBOX_YES_NO,"Create Non-Existent Key","The key that contains the \"Media Sensing\" setting \nis not present in the registry.  This would typically \nindicate that the system is not running Windows 2000.  \nThis fix is only necessary for machines that are \nrunning Windows 2000.  \n\nCreate the key?");
    if(reply==MBOX_CLICKED_YES)
    {
      DWORD disp;
      success=RegCreateKeyEx(hkey,"System\\CurrentControlSet\\Services\\Tcpip\\Parameters",0,"",REG_OPTION_NON_VOLATILE,KEY_SET_VALUE,NULL,&hsubkey,&disp);
      if(success!=ERROR_SUCCESS)
      {
        showErrorMessage(success);
        RegCloseKey(hkey);
        return 1;
      }
    }
    else
    {
      RegCloseKey(hkey);
      return 1;
    }
  }
  else if(success!=ERROR_SUCCESS)
  {
    showErrorMessage(success);
    RegCloseKey(hkey);
    return 1;
  }

  DWORD val=(enable)?0:1;
  DWORD size=sizeof(DWORD);
  success=RegSetValueEx(hsubkey,"DisableDHCPMediaSense",0,REG_DWORD,(CONST BYTE*)&val,size);
  if(success!=ERROR_SUCCESS)
  {
    showErrorMessage(success);
    RegCloseKey(hsubkey);
    RegCloseKey(hkey);
    return 1;
  }
  
  if(val==0)
  {
    value=VALUE_ENABLED;
    enable=FALSE;
  }
  else
  {
    value=VALUE_DISABLED;
    enable=TRUE;
  }

  RegFlushKey(hsubkey);
  RegCloseKey(hsubkey);
  RegCloseKey(hkey);

  //Will probably want to reboot a remote host
  if(host!=LOCALHOST)
  {
    if(rebootSystem())
      return 1;
  }

  dirty=TRUE;

  return 1;
}

long MediSen::onUpdApply(FXObject* sender,FXSelector,void*)
{
  if(value!=VALUE_DISABLED&&!enable)
    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
  else if(value==VALUE_DISABLED&&enable)
    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
  else
    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);

  return 1;
}

long MediSen::onCmdHelp(FXObject*,FXSelector,void*)
{
  HelpDialog help(this,"MediSen Help");
  help.execute();
  return 1;
}

long MediSen::onCmdAbout(FXObject*,FXSelector,void*)
{
  FXString msg="MediSen - Windows 2000 Media Sensing Manager\nVersion 1.0\n\n"\
"Copyright (C) 2001 Dustin Graves (dgraves@computer.org)\n\n"\
"MediSen comes with ABSOLUTELY NO WARRANTY.\n"\
"This is free software, and you are welcome to redistribute it under\n"\
"certain conditions; for details see the GNU General Public License:\n"\
"http://www.gnu.org/copyleft/gpl.html\n\n"\
"This software uses the FOX Platform Independent GUI Toolkit Library.\n"\
"The FOX Library is Copyright (C) 1997,2000,2001 Jeroen van der Zijp and is\n"\
"available freely under the GNU Lesser Public License at the following site:\n"\
"http://www.cfdrc.com/FOX/fox.html";

  FXMessageBox about(this,"About MediSen",msg,bigIcon,MBOX_OK|DECOR_TITLE|DECOR_BORDER);
  about.execute();  

  return 1;
}

long MediSen::onPopupMenu(FXObject* sender,FXSelector sel,void* ptr)
{
  FXString str;
  switch(SELID(sel))
  {
  case ID_STATUS:
    str="Media Sensing Status\n\n"\
        "Displays one of the following three status\n"\
        "messages to indicate the Media Sensing\n"\
        "status for a given host:\n\n"\
        "  1.  Media Sensing feature is not currently disabled for <hostname>\n"\
        "  2.  Media Sensing feature is currently enabled for <hostname>\n"\
        "  3.  Media Sensing feature is currently disabled for <hostname>\n\n"\
        "<hostname> indicates the name of the host that\n"\
        "MediSen is connected to.\n\n"\
        "Option number one indicates that there is\n"\
        "currently no setting in the registry to either\n"
        "enable or disable the Media Sensing feature.\n"\
        "In this case the Media Sensing feature will\n"
        "default to on.\n\n"\
        "Option number two indicates that the Media Sensing\n"\
        "registry value is set to true, indicating that the\n"\
        "Media Sensing feature is currently on.\n\n"\
        "Option number three indicates that the Media Sensing\n"\
        "registry value is set to false, indicating that the\n"\
        "Media Sensing feature is currently off.";
      break;
  case ID_MSENABLE:
    str="Enable Media Sensing\n\n"\
        "This radio button is used to enable\n"\
        "the Windows 2000 Media Sensing feature.\n"\
        "Select this option and click the 'Apply'\n"\
        "button in order to activate the change.";
    break;
  case ID_MSDISABLE:
    str="Disable Media Sensing\n\n"\
        "This radio button is used to disable\n"\
        "the Windows 2000 Media Sensing feature.\n"\
        "Select this option and click the 'Apply'\n"\
        "button in order to activate the change.";
    break;
  case ID_APPLY:
    str="Apply Changes to Media Sensing\n\n"\
        "This button is used to apply changes to\n"\
        "the Windows 2000 Media Sensing feature.\n"\
        "After selecing an option above, click this\n"\
        "button in order to activate the change.\n"\
        "If this button is disabled, the current\n"\
        "selection matches the current state of the\n"\
        "system and no changes need to be applied.";
    break;
  case ID_CANCEL:
    str="Exit Application\n\n"\
        "This button is used to exit the\n"\
        "application.  If changes have been made\n"\
        "to the sytstem's Media Sensing setting\n"\
        "a dialog box providing the option to reboot\n"\
        "the system will be displayed.";
    break;
  }

  if(!str.empty())
  {
    FXEvent* event=(FXEvent*)ptr;
    FXMenuPane pane(this);
    new FXMenuCommand(&pane,"&Huh?",NULL,this,ID_CLARIFY);

    clarify=FALSE;

    pane.create();
    pane.popup(NULL,event->root_x,event->root_y);
    getApp()->runModalWhileShown(&pane);

    if(clarify)
    {
      FXMenuPane clarification(this,FRAME_LINE);

      FXButton* button=new FXButton(&clarification,str,NULL,&clarification,FXWindow::ID_HIDE,JUSTIFY_LEFT);
      button->setFont(getApp()->getNormalFont());
      button->setTextColor(getApp()->getForeColor());
      button->setBackColor(FXRGB(255,255,192));

      clarification.create();
      clarification.popup(NULL,event->root_x,event->root_y);
      getApp()->runModalWhileShown(&clarification);
    }
  }
  return 1;
}

long MediSen::onCmdClarify(FXObject*,FXSelector,void*)
{
  clarify=TRUE;
  return 1;
}

int main(int argc,char** argv)
{
  FXApp app("MediSen","SystemUtility");
  app.init(argc,argv);
  new MediSen(&app);
  app.create();
  return app.run();
}
